#include <sys/ioctl.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <time.h>

#define EXT3_IOC_FSCK		_IOW('f', 11, long)
#define EXT3_IOC_FSCK_TEST_REQUEST _IOW('f', 12, long)
#define EXT3_IOC_FSCK_TEST_UMOUNT _IOW('f', 13, long)


typedef struct icefs_fsck_user_state {
    int cube_no;
    int flag; // 0: waiting for fsck request; 1: finished a fsck request; <0: there is an error during fsck
} icefs_fsck_user_state_t;

int main(int argc, char *argv[]) {
    int fd;
    char cmd[256];
    char *dev_file;
    char *ioctl_file;
    char *fsck_path;
    int ret=0;

    icefs_fsck_user_state_t request={0};

    if(argc<2) {
	printf("./fsck_daemon dev_file ioctl_file fsck_path\n");
	return 0;	
    }
    else {
	dev_file = argv[1];
	ioctl_file = argv[2];
	fsck_path = argv[3];
	printf("dev: %s\nioctl: %s\nfsck: %s\n", dev_file, ioctl_file, fsck_path);
    }	
    
    fd = open(ioctl_file, O_RDONLY);
    if(fd<0){
	perror("ERROR: Unable to open ioctl file\n");
	exit(EXIT_FAILURE);
    }


    do {
	printf("EXT3_IOC_FSCK enter\n");
	ret = ioctl(fd, EXT3_IOC_FSCK, &request);	
	printf("EXT3_IOC_FSCK return\n");
    
	printf("EXT3_IOC_FSCK get a request: cube %d, flag %d, umount? %d\n", request.cube_no, request.flag, ret);

	if(ret==1)
	    // umount
	    break;
	else {
	
	    // ========================
	    // run fsck

	    struct timeval tstart, tend;
	    unsigned long tt=0;

	    system("echo 'START FSCK'");
	    sprintf(cmd, "%s/e2fsck -p -f -z %d %s", fsck_path, request.cube_no, dev_file);


	    gettimeofday(&tstart, NULL);      
	    system(cmd);
	    gettimeofday(&tend, NULL);      

	    tt = ((tend.tv_sec * 1000000 + tend.tv_usec) - (tstart.tv_sec * 1000000 + tstart.tv_usec));

	    printf("FSCK time: %f\n", ((float)tt)/1000000.0);

	    
	    system("echo 'END FSCK'");
	    // ========================

	    // TODO: return any error back to the file system
	    request.flag = 1;
	}
    } while(ret==0);

    printf("EXIT due to umount\n");

    close(fd);

    return 0;
}
